@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+from statistics import median_high
 from typing import Optional
 from typing import Tuple
 from typing import Union
@@ -15,8 +16,7 @@ from ultron.optimize.constraints import LinearConstraints
 from ultron.optimize.linearbuilder import linear_builder
 from ultron.optimize.longshortbulder import long_short_builder
 from ultron.optimize.meanvariancebuilder import mean_variance_builder
-from ultron.factor.experimental.portfolio.meanvariancebuilder import mean_variance_builder as f_mean_variance_builder
-from ultron.factor.experimental.portfolio.linearbuilder import linear_builder as f_linear_builder
+from ultron.optimize.meanvariancebuilder import target_vol_builder
 from ultron.factor.analysis.simplesettle import simple_settle
 
 
@@ -87,6 +87,7 @@ def er_portfolio_analysis(er: np.ndarray,
         return lbound, ubound, cons_exp, risk_lbound, risk_ubound
 
     if method == 'risk_neutral':
+        print(method)
         lbound, ubound, cons_exp, risk_lbound, risk_ubound = create_constraints(benchmark, **kwargs)
 
         turn_over_target = kwargs.get('turn_over_target')
@@ -102,40 +103,12 @@ def er_portfolio_analysis(er: np.ndarray,
         if status not in ("optimal", "optimal_inaccurate"):
             raise ValueError('linear programming optimizer in status: {0}'.format(status))
 
+    elif method == 'rank':
+        weights = rank_build(er, use_rank=kwargs['use_rank'],
+                             masks=is_tradable).flatten() * benchmark.sum() / kwargs[
+                      'use_rank']
     elif method == 'ls' or method == 'long_short':
         weights = long_short_builder(er).flatten()
-    if method == 'f_risk_neutral':
-        lbound, ubound, cons_exp, risk_lbound, risk_ubound = create_constraints(benchmark, **kwargs)
-        turn_over_target = kwargs.get('turn_over_target')
-        current_position = kwargs.get('current_position')
-        status, _, weights = f_linear_builder(er,
-                                            risk_constraints=cons_exp,
-                                            lbound=lbound,
-                                            ubound=ubound,
-                                            risk_target=(risk_lbound, risk_ubound),
-                                            turn_over_target=turn_over_target,
-                                            current_position=current_position)
-        if status not in ("optimal", "optimal_inaccurate"):
-            raise ValueError('linear programming optimizer in status: {0}'.format(status))
-
-    elif method == 'fmv':
-        lbound = kwargs['lbound']
-        ubound = kwargs['ubound']
-        risk_model = kwargs['risk_model']
-        turn_over_target = kwargs.get('turn_over_target')
-        current_position = kwargs.get('current_position')
-        target_vol = kwargs.get('target_vol')
-        status, _, weights = mean_variance_builder(er=er,
-                                                   risk_model=risk_model,
-                                                   current_position=current_position,
-                                                   turn_over_target=turn_over_target,
-                                                   vol_target=target_vol,
-                                                   lbound=lbound,
-                                                   ubound=ubound
-                                                   )
-        if status != 'optimal':
-            raise ValueError('mean variance optimizer in status: {0}'.format(status))
-
     elif method == 'mv' or method == 'mean_variance':
         lbound, ubound, cons_exp, risk_lbound, risk_ubound = create_constraints(benchmark, **kwargs)
         risk_model = kwargs['risk_model']
@@ -156,6 +129,23 @@ def er_portfolio_analysis(er: np.ndarray,
         if status != 'optimal':
             raise ValueError('mean variance optimizer in status: {0}'.format(status))
 
+    elif method == 'tv' or method == 'target_vol':
+        lbound, ubound, cons_exp, risk_lbound, risk_ubound = create_constraints(benchmark, **kwargs)
+        risk_model = kwargs['risk_model']
+
+        if 'target_vol' in kwargs:
+            target_vol = kwargs['target_vol']
+        else:
+            target_vol = 1.
+
+        status, _, weights = target_vol_builder(er,
+                                                risk_model=risk_model,
+                                                bm=benchmark,
+                                                lbound=lbound,
+                                                ubound=ubound,
+                                                risk_exposure=cons_exp,
+                                                risk_target=(risk_lbound, risk_ubound),
+                                                vol_target=target_vol)
     else:
         raise ValueError("Unknown building type ({0})".format(method))
 
